#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

import subprocess
from tempfile import mkdtemp
from os import chdir
from os.path import join
from auto_abi_checker.utils import _check_call, error, info, subinfo, main_step_info


class ABIExecutor():
    def __init__(self, compilation_flags=''):
        self.bin = 'abi-compliance-checker'
        self.ws = mkdtemp()
        self.ws_abi_dump = join(self.ws, 'abi_dumps')
        self.ws_report = join(self.ws, 'compat_reports', 'X_to_X')
        self.report_name = 'test_name_report'
        self.compilation_flags = compilation_flags
        self.no_fail_if_emtpy = False
        self.empty_objects_found = False

    def run(self, orig_src, new_src, report_dir='', no_fail_if_emtpy=False):
        main_step_info("Run abichecker")
        # Use orig value as report name
        self.report_name = orig_src.name
        self.no_fail_if_emtpy = no_fail_if_emtpy
        # if compilation_flags is set respect the value
        if not self.compilation_flags:
            self.compilation_flags = self.get_compilation_flags(
                    orig_src, new_src)
        chdir(self.ws)
        if report_dir:
            self.ws_report = report_dir
        if self.dump(orig_src) != 0:
            error("ABI Dump from " + str(orig_src) + " failed")
        if self.dump(new_src) !=0:
            error("ABI Dump from " + str(new_src) + " failed")
        self.generate_report(orig_src, new_src)

    def get_compilation_flags(self, orig_src, new_src):
        r = list(orig_src.compilation_flags)
        r.extend(x for x in new_src.compilation_flags if x not in r)
        return " ".join(r)

    def get_dump_file(self, src_class):
        return join(self.ws_abi_dump,
                    src_class.name, 'X', 'ABI.dump')

    def get_compat_report_file(self):
        return join(self.ws_report, 'compat_report.html')

    def dump(self, src_class):
        cmd = [self.bin,
               '-l', src_class.name,
               '-dump', src_class.ws_files,
               '-gcc-options', self.compilation_flags]
        print(" - Run '%s'" % ' '.join(cmd))
        try:
            subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            if "library objects are not found".encode('utf-8') in e.output:
                self.empty_objects_found = True
                if self.no_fail_if_emtpy:
                    subinfo("Skip error on no object found")
                    return 0
                error("No lib objects found in " + str(src_class))
            return 1

        return 0

    def generate_report(self, orig_src, new_src):
        if self.empty_objects_found:
            main_step_info("No report generated since empty dumps were found")
            return 0

        result = _check_call([self.bin,
                             '-l', self.report_name,
                             '-report-path', self.get_compat_report_file(),
                             '-old', self.get_dump_file(orig_src),
                             '-new', self.get_dump_file(new_src)])

        if result != 0:
            return result

        main_step_info("Generated: " + self.get_compat_report_file())



#!python

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0
"""
Usage:
        auto-abi --orig-type <orig-type> --orig <orig> --new-type <new-type> --new <new> [--report-dir <dir>] [--no-fail-if-empty] [--never-fail] [--display-exec-time]
        auto-abi (-h | --help)
        auto-abi --version

orig-type:
        local-dir               Code to check is a local directory
                                Use full path to installation dir
        osrf-pkg                Code to check is a OSRF package
                                Use repository name for value (i.e. sdformat7)
        ros-pkg                 Use the ROS package name. It can be fully qualified
                                (i.e ros-melodic-gazebo-dev) or a ROS name
                                (i.e gazebo_dev). Multiple packages are supported,
                                comma separated.
        ros-repo                Code to check is a ROS repository
                                Use repository name for value (i.e gazebo_ros_pkgs)

Options:
        -h --help               Show this screen
        --version               Show auto-abi version
        --report_dir <dir>      Generate compat report in <dir>
        --no-fail-if-empty      Return 0 if the abi checker does not found object files
        --never-fail            Return 0 always even in the presence of errors
        --display-exec-time     Show the execution time of this script
"""

import datetime
from sys import stderr
from time import time
from docopt import docopt
from auto_abi_checker.generator import SrcGenerator
from auto_abi_checker.abi_executor import ABIExecutor
from auto_abi_checker.utils import error, AppError, info


def normalize_args(args):
    orig_type = args["<orig-type>"]
    orig_value = args["<orig>"]

    new_type = args["<new-type>"]
    new_value = args["<new>"]

    report_dir = args["<dir>"]
    no_fail_if_emtpy = args["--no-fail-if-empty"]
    never_fail = args["--never-fail"]
    display_exec_time = args["--display-exec-time"]

    return orig_type, orig_value, new_type, new_value, report_dir, no_fail_if_emtpy, never_fail, display_exec_time


def check_type(value_type):
    if (value_type == 'ros-repo' or
        value_type == 'ros-pkg' or
        value_type == 'osrf-pkg' or
        value_type == 'local-dir'):
        True
    else:
        error("Unknow type " + value_type)


def validate_input(args):
    orig_type, orig_value, new_type, new_value, report_dir, no_fail_if_emtpy, never_fail, display_exec_time = args

    if (check_type(orig_type) and check_type(new_type)):
        return True

    return False


def process_input(args):
    orig_type, orig_value, new_type, new_value, report_dir, no_fail_if_emtpy, never_fail, display_exec_time = args

    if display_exec_time:
        start = time()

    try:
        generator = SrcGenerator()
        src_gen = generator.generate(orig_type, 'orig')
        src_gen.run(orig_value)
        new_gen = generator.generate(new_type, 'new')
        new_gen.run(new_value)

        abi_exe = ABIExecutor()
        abi_exe.run(src_gen, new_gen, report_dir, no_fail_if_emtpy)

        if display_exec_time:
            exec_time = time() - start
            info("Execution time: " + str(datetime.timedelta(seconds=exec_time)))
    except Exception as e:
        if never_fail:
            print("[warn] detected errors but return 0 since --never-fail option is enabled", file=stderr)
        else:
            exit(-1)


def main():
    try:
        args = normalize_args(docopt(__doc__, version="auto-abi 0.1.0"))
        validate_input(args)
        process_input(args)
    except KeyboardInterrupt:
        print("auto-abi was stopped with a Keyboard Interrupt.\n")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

from tempfile import mkdtemp
from glob import glob
from os.path import join
from auto_abi_checker.utils import main_step_info


class SrcBase:
    def __init__(self, name):
        self.name = name
        self.ws = mkdtemp()
        self.ws_files = join(self.ws, 'files')
        self.compilation_flags = []
        main_step_info("Init " + self.name + " -::- " + "workspace: " + self.ws)

    def __str__(self):
        return "'" + self.name + "' at " + str(self.ws)

    def list_files(self, pattern):
        return glob(self.ws + '/' + pattern)

    def get_cmd_compilation_flags(self):
        return " ".join(self.compilation_flags)



#!/usr/bin/env python3

# Copyright 2018 Open Robotics
# Licensed under the Apache License, Version 2.0

from subprocess import check_call
from sys import stderr


def main_step_info(msg):
    print("* %s" % msg)


def info(msg):
    print(" - %s" % msg)


def subinfo(msg):
    print("   - %s" % msg)


def _check_call(cmd):
    info("Run " + " ".join(cmd))

    check_call(cmd)
    return 0


class AppError(Exception):
    pass


def error(msg):
    print("\n [err] " + msg + "\n", file=stderr)
    raise AppError()


def warn(msg):
    print("\n [warn] " + msg + "\n", file=stderr)


def comma_list_to_array(str_input):
    if "," in str_input:
        return str_input.split(",")

    return [str_input]
